#!/usr/bin/perl -T
# $Id: ajax.cgi,v 1.33 2017/09/01 05:58:06 sqko Exp $
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use RFC::Furnace;
use LWP::UserAgent;
use SemaforeFile;
print header();
my $ac = param('action');
unless ($ac)
{
    ## To do: print out all avaliable function description including all nescessarry arguments/parameters
    print "";
    exit;
}

if ($ac eq 'get_furnaces')
{
    my @list = get_cv('celltest','rigs');
    if (@list)
    {
	print "OK\n";
	print join("\n",@list);
    }
    else
    {
	print "Warning: Could not find any furnaces!\n";
    }
    exit;
}

if ($ac eq 'get_furnaces_equipid')
{
    my @list = get_cv('celltest','rigs');
    if (@list)
    {
	print "OK\n";
	foreach my $r (@list)
	{
	    next unless ($r && $r=~ /\d+/);
	    my $f = RFC::Rig->new($r);
	    my $id = $f->get_cv('main','equipmentid');
	    $id = '' unless ($id);
	    print "$r\t$id\n";
	}
    }
    else
    {
	print "Warning: Could not find any furnaces!\n";
    }
    exit;
}

if ($ac eq 'get_listservers')
{
    my @list = ();
    my @listservers = ();
    my $listserver = get_cv('servers','listserver');
    if ($listserver)
    {
        chomp $listserver;
        my $str= &RFC::Main::report_client($listserver,'list_servers');
        my @l = split(/\t/,$str) if ($str && $str !~ /ERROR/);
        foreach (@l)
        {
            if (/([\w\.\-]+)/)
            {
                push @listservers,$1;
            }
        }
    }
    if (@listservers)
    {
	print "OK\n";
	print join("\n",@listservers);
    }
    else
    {
	print "Warning: No list servers found!\n";
    }
    exit;
}

if ($ac eq 'get_servers')
{
    my @list = ();
    my $listserver = get_cv('servers','listserver');
    if ($listserver)
    {
        chomp $listserver;
        my $str= &RFC::Main::report_client($listserver,'servers');
        my @l = split(/\t/,$str) if ($str && $str !~ /ERROR/);
        foreach (@l)
        {
            if (/([\w\.\-]+)/)
            {
                push @list,$1;
            }
        }
    }
    if (@list)
    {
	print "OK\n";
	print join("\n",@list);
    }
    else
    {
	print "Warning: No servers found!\n";
    }
    exit;
}

if ($ac eq 'checkuser')
{
    my $uid = &login_ok();
    if ($uid)
    {
	print "OK\n$uid\n";
    }
    else
    {
	print "Error: User not authenticated! (perhaps session timeout?)\n";
    }
    exit;
}

if ($ac eq 'new_session')
{
    my ($user,$pass) = &RFC::RFCCGI::get_id();
    my $md5 = RFC::Main::passwd_client('new_session',$user,$pass);
    my $uid = RFC::Main::passwd_client('checkuser',$user,$md5);
    if ($uid)
    {
	print "OK\n";
	print "$uid\n";
	print "$md5\n";
    }
    else
    {
	print "User not authenticated!\n";
    }
    exit;
}

if ($ac eq 'get_tube_elements')
{
    my $tube = param('tube');
    my @list = RFC::Furnace::get_tube_elements($tube);
    print "OK\n";
    print join(',',@list),"\n";
    exit;
}

if ($ac eq 'check_atoms')
{
    my $str = get_CGI_value('comp');
    if ($str =~ /^[\w\,\s]+$/)
    {
	my @elements = RFC::Furnace::get_element_data();
	if (&check_elements($str))
	{
	    print "OK\nConfirm chosen elements:\n";
	    foreach my $n (split(/[\s\,\t]+/,$str))
	    {
		foreach my $e (@elements)
		{
		    if ($$e{'short'} eq $n)
		    {
			print $$e{'name'},"\n";
			last;
		    }
		}
	    }
	}
	else
	{
	    ### Find offending elements in list
	    my @lst = ();
	    foreach my $n (split(/[\s\,\t]+/,$str))
	    {
		my $found = 0;
		foreach my $e (@elements)
		{
		    if ($$e{'short'} eq $n)
		    {
			$found = 1;
			last;
		    }
		}
		push @lst,$n unless ($found)
	    }
	    print 'Warning: Element(s) '.join(', ',@lst)." was not recognised!\n";
	}
    }
    else
    {
        print "Illegal characters found in elemental composition! Only letters, spaces and commas allowed!\n";
    }
    exit;
}

my $r = param('rig');
if ($r && $r =~ /(\d+)/)
{
    $r = $1;
}
else
{
    $r = 0;
}
unless ($r)
{
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Furnace->new($r);

if ($ac eq 'list_tubes_furnace')
{
    my @list = $rig->list_tubes();
    print "OK\n";
    print join(',',@list),"\n";
    exit;
}

if ($ac eq 'add_program')
{
    my $prog = param('program');
    my @lines = split(/NEWLINE/,$prog);
    $rig->append_program(@lines);
    my ($png,$time) = $rig->get_plot();
    if ($png && $png =~ /(\/rig.*)/)
    {
	print "OK\n";
	print $1,"\n";
	printf ("%.1f",$time/60);
    }
    else
    {
	print "Warning: No png could be created!\n";
    }
    exit;
}

if ($ac eq 'plot_program')
{
    my $runnr = &get_CGI_value_clean('runnr');
    my ($png,$time) = $rig->get_plot($runnr,'force');
    if ($png && $png =~ /(\/rig.*)/)
    {
	print "OK\n";
	print $1,"\n";
	printf ("%.1f",$time/60);
    }
    else
    {
	print "Warning: No png could be created!\n";
    }
    exit;
}

if ($ac eq 'is_running')
{
    print "OK\n";
    if ($rig->is_running)
    {
	print "1\n";
    }
    else
    {
	print "0\n";
    }
    exit;
}

if ($ac eq 'get_furnace_elements')
{
    print join("\n",'OK',$rig->list_elements());
    exit;
}

unless (login_ok())
{
    print "You are not authorised!\n";
    exit;
}

my $user = &get_user();

unless (RFC::RFCCGI::rig_auth())
{
    print "You are not authorised to run $rig\n";
    exit;
}

if ($ac eq 'add_log')
{
    my $msg = param('msg');
    my $equipid = $rig->rig->get_cv('main','equipmentid');
    print $equipid,"\n";
    &remote_log($equipid,$msg) if ($equipid);
    $rig->proglog($msg,$user);
}

if ($ac eq 'add_tube_element')
{
    my $tube = param('tube');
    my $element = param('element');
    my $res = &RFC::Furnace::add_tube_element($tube,$element);
    if ($res)
    {
	my $str = "User $user added element $element to tube $tube";
	my $equipid = $rig->rig->get_cv('main','equipmentid');
	&remote_log($equipid,$str) if ($equipid);
	$rig->proglog($str,$user);	
    }
    print "OK";
    exit;
}

if ($ac eq 'del_tube_element')
{
    my $tube = param('tube');
    my $element = param('element');
    my $res = &RFC::Furnace::del_tube_element($tube,$element);
    if ($res)
    {
	my $str = "User $user removed element $element from tube $tube";
	my $equipid = $rig->rig->get_cv('main','equipmentid');
	&remote_log($equipid,$str) if ($equipid);
	$rig->proglog($str,$user);
    }
    print "OK";
    exit;
}

if ($ac eq 'get_info')
{
    my $runnr = param('runnr');
    if ($runnr && $runnr =~/(\d+)/)
    {
	$runnr = $1;
    }
    else
    {
	$runnr = $rig->runnr();
    }
    print join("\n",$rig->get_info($runnr));
    exit;
}

if ($ac eq 'get_proglog')
{
    my $runnr = param('runnr');
    if ($runnr && $runnr =~/(\d+)/)
    {
	$runnr = $1;
    }
    else
    {
	$runnr = $rig->runnr();
    }
    print join("\n",$rig->get_proglog($runnr));
    exit;
}

if ($ac eq 'add_info')
{
    my $info = param('comments');
    exit unless ($info);
    $info =~ s/\\n/\n/g;
    my $header = 'Added by '.get_user().' on '.localtime().':';
    $rig->set_info($header,$info);
    print join("\n",$rig->get_info());
    exit;
}

unless (&RFC::Furnace::can_start_furnace())
{
    print "Warning: The system you are attempting to start and/or stop a furnace from\nis not on the approved list!";
    exit;
}


if ($ac eq 'stop_program')
{
    my $runid = 'Furnace '.$rig->name.' run '.$rig->runnr().': ';
    $rig->proglog("Program terminated by user $user!");
    $rig->stop_program();
    sleep(2);
    print "Furnace program terminated!";
    my $equipid = $rig->rig->get_cv('main','equipmentid');
    &remote_log($equipid,$runid."Furnace run terminated!") if ($equipid);
    exit;
}

if ($ac eq 'start_program')
{
    eval {
	my $tube = param('tube');
	if ($tube && $tube > 0)
	{
	    my $tube_ok = 0;
	    my @tubes = $rig->list_tubes();
	    foreach (@tubes)
	    {
		$tube_ok = 1 if ($_ == $tube);
	    }
	    die "Furnace tube '$tube' is not marked as a valib tube for furnace $rig!" unless ($tube_ok);
	}
	my $prog = param('program');
	my @lines = split(/NEWLINE/,$prog);
	$rig->append_program(@lines);
	my $fn = $RFC::Header::publicdir.'rig'.$r.'/furnace_runs/program.txt';
	my $f = SemaforeFile->new($fn,'/tmp/start_furnace.lock');
	$f->lock_ex();
	$f->writeline($rig->program_to_file);
	my $res = RFC::Main::CGI_client($rig->name,'start_furnace',$fn,$user);
	$f->unlock();
	sleep(5);
	my $info = param('info');
	$info = '' unless ($info);
	my @new_elements = ();
	my $elements = param('comp');
	if ($tube)
	{
	    my @old_elements = split(/,/,param('tube_elements'));
	    unless (@old_elements)
	    {
		my @elements = &RFC::Furnace::get_element_data();
		foreach (&RFC::Furnace::get_tube_elements($tube))
		{
		    push @old_elements, $elements[$_]{'short'};
		}
	    }
	    if ($elements)
	    {
		foreach my $n (split(/[\s\,\t]+/,$elements))
		{
		    &RFC::Furnace::add_tube_element($tube,$n);
		    my $found = 0;
		    foreach (@old_elements)
		    {
			next unless ($_ eq $n);
			$found = 1;
			last;
		    }
		    push @new_elements,$n unless ($found);
		}	    
	    }	    
	}
	else
	{
	    if ($elements)
	    {
		foreach my $n (split(/[\s\,\t]+/,$elements))
		{
		    my $res = $rig->add_element($n);
		    push @new_elements,$n if ($res);
		}	    
	    }
	}
	my $start = 'Program started: '.localtime();
	my $user = 'Started by: '.get_user();
	@lines  = ($start,$user,split(/NEWLINE/,$info));
	$rig->set_info(@lines);
	my $runid = 'Furnace '.$rig->name.' run '.$rig->runnr().': ';
	my $str =  $runid."\n".join("\n",@lines);
	my $equipid = $rig->rig->get_cv('main','equipmentid');
	&remote_log($equipid,$str) if ($equipid);
	if (@new_elements)
	{
	    my $subj;
	    my $str;
	    my $u = &get_user();
	    if ($tube)
	    {
		$str = "Warning: user '$u' has started $rig with the following elements not previously used in tube $tube: ".join(', ',@new_elements);
		$subj = "Warning potential contamination issue for tube $tube";		
	    }
	    else
	    {
		$str = "Warning: user '$u' has started $rig with the following elements not previously used in this furnace: ".join(', ',@new_elements);
		$subj = "Warning potential contamination issue for $rig";
	    }
	    $rig->rig->warning_mail('Furnace_elements',$str,$subj);
	    sleep(1);
	    $rig->rig->unset_warning('Furnace_elements');
	    if ($tube)
	    {
		print "Warning: You have started $rig with elements not previously seen for this tube ($tube): ".join(', ',@new_elements),"\n";
		print "It is strongly advised to stop the furnace immediately and contact the furnace administrators.\n";
		print "Note that an email has already been sent to the administraors documenting this issue!\n";		
	    }
	    else
	    {
		print "Warning: You have started $rig with elements not previously seen: ".join(', ',@new_elements),"\n";
		print "It is strongly advised to stop the furnace immediately and contact the furnace administrators.\n";
		print "Note that an email has already been sent to the administraors documenting this issue!\n";
	    }
	    &remote_log($equipid,$runid.$str) if ($equipid);
	}
	else
	{
	    print "Furnace $rig started!";	    
	}
	### To do: what should we do with the list of new elements? ###
    };
    if ($@)
    {
	print "Warning: $@";
    }
    exit;
}

sub check_elements($)
{
    my $str = shift;
    my @elements = RFC::Furnace::get_element_data();
    foreach my $i (split(/\s*\,\s*/,$str))
    {
	my $found = 0;
	foreach my $e (@elements)
	{
	    if ($i eq $$e{'short'})
	    {
		$found = 1;
		last;
	    }
	}
	return 0 unless ($found);
    }
    return 1;
}


sub remote_log($$)
{
    my $eqid = shift;
    my $msg = shift;
    my $eqadr = &RFC::Main::get_cv('global','remote_equipment_database');
    return unless ($eqadr);
    my $ua = LWP::UserAgent->new();
    $ua->timeout(60);
    my $url = $eqadr;
    my $query = '?name='.&get_user().'&pass='.&get_pass().'&action=new_maint_log&type=Furnacelog&equipid='.$eqid.'&description='.$msg;
    my $request = new HTTP::Request('GET',$url.$query);
    my $response = $ua->request($request);
    return if ($response->is_success);
    &errorlog("Could not append mesage to remote message log for rig $rig!");
}
