#!/usr/bin/perl -T
# $Id: misc_conf.cgi,v 1.6 2017/09/01 09:19:07 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2017              ###
###                                                      ###
############################################################
############################################################
use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::Furnace;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use SemaforeFile;
my $acmode = get_cv('global','access_mode');
$acmode = 'celltest' unless ($acmode);

my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my @tubes = ();
my $furnace;
eval {
    $furnace = RFC::Furnace->new($rig);
    @tubes = $furnace->list_tubes();
};

my $equipid = $rig->get_cv('main','equipmentid');
$equipid = 0 unless ($equipid);
my $eqadr = get_cv('global','remote_equipment_database');
$eqadr = '' unless ($eqadr);
my $logaddr = get_cv('global','logbook_url');
my $readmode = undef;
my $usermode = undef;
my $modifymode = undef;
my $eqid = $rig->get_cv('main','equipmentid');
my $remote = $rig->get_cv('main','authorisation');
if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end('../RFCfurnace/manual.pdf');
}
else
{
    if (login_ok())
    {
        $readmode = user_auth($acmode,'r');
        $usermode = user_auth($acmode,'c');
        $modifymode = user_auth($acmode,'m');
        if (RFC::RFCCGI::rig_mode())
        {
            ####  Read and use mode deprecated after initialisation of rig authentificati on ####
	    $readmode = 1;
	    $usermode = 1;
	    $modifymode = RFC::RFCCGI::rig_cert() unless ($modifymode);
        }
        unless (RFC::RFCCGI::rig_auth())
        {
            $readmode = 0;
            $usermode = 0;
            $modifymode = 0;
        }
        &print_header("Main page for furnace ".$rig->name);
        print '<link rel="stylesheet" type="text/css" href="/RFCfurnace/Furnace.css" />',"\n";
        print '<script src="/RFCfurnace/Furnace.js" type="text/javascript"></script>',"\n";
        top_nav_bar_start;
	top_no_button;
	if (@tubes)
	{
	    top_no_button;
	}
	else
	{
	    print '<td>';
	    print button(-name=>'Manage chemical element exposure',-class=>'s4',-onclick=>"pick_elements_furnace(document.getElementById('f_elements').value)");
	    print '</td>';
	}
	top_no_button;
        tab_newrow;
        if (get_CGI_value('action'))
	{
            top_no_button;
        }
        else
        {
	    &js_back();
	    top_js_return('#006600','white');	    
	}
        if ($modifymode)
        {
            top_nav_bar_button('../celltest/rig_conf.cgi','','','Manual configuration','s1','rig,'.$rig->name.',1');
        }
        else
        {
            top_no_button;
        }
        if ($modifymode || $usermode)
        {
            top_nav_bar_button('../celltest/rig_setup.cgi','','',"$rig configuration",'s3','rig,'.$rig->name.',1');
        }
        else
        {
            top_no_button;
        }
        top_nav_bar_button('furnace.cgi','','',"To furnace ".$rig->name." main page",'s5','rig,'.$rig->name.',1');
        top_nav_bar_button('main.cgi','','','To menu','s4');
        end_top_bar;
        param(-name=>'action',-value=>'')  unless (defined(param('action')));
        $_ = param('action');
      CASE:
        {
	    /set_elements/ and do {&set_elements();};
            ### Default response ####
            &insert_data_form;
        }
        print_end;
    }
    else
    {
        &not_auth;
    }
}

exit;

sub set_elements()
{
    return unless ($furnace);
    my $user = &get_user();
    my $str = param('f_elements');
    return unless ($str);
    my @list = ();
    my @elements = RFC::Furnace::get_element_data();
    foreach my $n (split(/\s*\,\s*/,$str))
    {
        foreach my $e (@elements)
        {
            next unless ($$e{'short'} eq $n);
            push @list,$$e{'Z'};
            last;
        }
    }
    my @curr = $furnace->list_elements();
    ### ADD missing ###            
    foreach (@list)
    {
        $furnace->add_element($_,$user);
    }
    ## Remove deleted ##                                    
  OUTER:
    foreach my $o (@curr)
    {
	foreach my $i (@list)
        {
            next OUTER if ($i == $o);
	}
        $furnace->del_element($o,$user);
    }
}



sub insert_data_form
{
    print '<center>';
    print '<table cellspacing=5 cellpadding=5 class="tabel"><tr>';
    print '<td width="5%"></td>';
    print '<td width="15%"></td>';
    print '<td width="25%"></td>';
    print '<td width="5%"></td>';
    print '<td width="15%"></td>';
    print '<td width="25%"></td>';
    print '<td width="5%"></td>';
    print '<td width="5%"></td>';
    print '</tr>';
    my @tags = ();
    my @yn = ('','Yes','No');
    my @auth = ('local','remote');
    my @status = ('Ok','Out of order');
    foreach my $sec ('main')
    {
	my $heading = $sec;
	$heading =~ s/_/ /g;
	$heading =~s/(\w)(interface)/$1 $2/g;
	print '<tr><td height=20></td></tr>';
        print '<tr><td colspan=11 class="heading"> '.ucfirst(lc($heading)).' setup </td></tr>';
        my @cf = ('allowed_tubes','equipmentid','authorisation','warning_mails','status');
        my %setup = ();
        foreach my $tag (@cf)
        {
            $setup{$tag} = $rig->get_cv($sec,$tag);
        }
        my $count = 0;
        foreach my $key (@cf)
        {
            print '<tr><td></td>' unless ($count);
            my @l = ();
	    @l = @auth if ($key eq 'authorisation');
	    @l = @status if ($key eq 'status');
            my $tag = $key;
            print '<th>';
            $tag =~ s/_/ /g;
            print xss(ucfirst($tag));
            print '</th><td>';
            if (@l>1)
            {
		print popup_menu(-name=>$key,-values=>[(@l)],-class=>'textfield',-default=>$setup{$key},-force=>1,-onchange=>"change_val(this,'$sec')",-id=>$key);
            }
            else
            {
		print textfield(-name=>$key,-value=>$setup{$key},-class=>'textfield',-force=>1,-onchange=>"change_val(this,'$sec')",-id=>$key);
            }
            print '</th><th></th>';
            if ($count++ > 0)
            {
                $count = 0;
                print '</tr>',"\n";
            }
        }
    }
    print '</table>';
    print p();
    my @list = ();
    my @elements = RFC::Furnace::get_element_data();
    foreach my $z ($furnace->list_elements())
    {
	foreach my $e (@elements)
	{
	    push @list,$$e{'short'} if ($$e{'Z'} == $z);
	}
    }
    print start_form(-name=>'set_elements');
    print '<th>';
    &action('set_elements');
    print hidden(-name=>'f_elements',-value=>join(',',@list),-force=>1,-id=>'f_elements');
    print_hidden_rig();
    print end_form();
    print '</center>';
    print <<END;

<script language="javascript">


function pick_elements_furnace(s)
{
    var myWindow=window.open('pick_elements.cgi?furnace=1&elements='+s,'windowRef','width=1050,height=550,scrollbars=1,resizable=1');
    myWindow.location.href = 'pick_elements.cgi?furnace=1&elements='+s;
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}

function set_temp_ok(t)
{
    alert(t);
    /* Sumbit the login info to reset logout timer */
    document.login_info.submit();
}

function set_val(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK" || arr[0] == "WARN")
    {
        if (arr[0] == "WARN")
        {
            alert(arr[3]);
        }
        var obj = document.getElementById(arr[1]);
        obj.disabled = false;
        obj.value = arr[2];
    }
    else
    {
        alert(arr[0]);
    }
    if (arr[1] == 'allowed_tubes')
    {
	document.login_info.submit();
    }
}

function change_val(element,section)
{
    var n = element.name;
    var val = element.value;
    element.disabled = true;
    run_ajax('../celltest/change_config.cgi','&action=change_config_value&section='+section+'&key='+n+'&value='+val,set_val);
}

</script>

END



}
